/*
 * Copyright (c) 2018 Nordic Semiconductor ASA
 *
 * SPDX-License-Identifier: LicenseRef-BSD-5-Clause-Nordic
 */

#include <zephyr.h>
#include <kernel_structs.h>
#include <stdio.h>
#include <string.h>
#include <gps.h>
#include <sensor.h>
#include <console.h>
#include <misc/reboot.h>
#include <logging/log_ctrl.h>
#if defined(CONFIG_BSD_LIBRARY)
#include <net/bsdlib.h>
#include <lte_lc.h>
#include <modem_info.h>
#endif /* CONFIG_BSD_LIBRARY */
#include <net/cloud.h>
#include <net/socket.h>
#include <nrf_cloud.h>

#include "cloud_codec.h"
#include "env_sensors.h"
#include "orientation_detector.h"
#include "ui.h"
#include "gps_controller.h"



#define MAXLINE 4096
#define HTTP_HOST "142.93.37.192"  
#define HTTP_PATH "/"
#define HTTP_PORT 8001
#define RECV_BUF_SIZE 8192
// my webserver: http://142.93.37.192:8001 

#define TEST_SIZE 5       


#define TEST_STRING  "HELLO FROM THINGY 91"

#define POST_TEMPLATE "POST %s? HTTP/1.1\r\n"\
		"Host: %s\r\n"\
		"Connection: keep-alive\r\n"\
	        "Content-type: application/x-www-form-urlencoded\r\n"\
		"Content-length: %d\r\n\r\n"\
		"%s"



struct k_timer my_timer;

bool read_exec = false;


char recv_buf[RECV_BUF_SIZE + 1];

int blocking_recv(int fd, u8_t *buf, u32_t size, u32_t flags)
{
	int err;
        int count = 0;

	do {
                count++;
                //printk("count: %d", count);
		err = recv(fd, buf, size, flags);
                //printk("count (under): %d", count);
	} while (err < 0 && errno == EAGAIN && (count<300));

	return err;
}

int blocking_send(int fd, u8_t *buf, u32_t size, u32_t flags)
{
	int err;

	do {
		err = send(fd, buf, size, flags);
	} while (err < 0 && errno == EAGAIN);

	return err;
}

int blocking_connect(int fd, struct sockaddr *local_addr, socklen_t len)
{
	int err;

	do {
		err = connect(fd, local_addr, len);
	} while (err < 0 && errno == EAGAIN);

	return err;
}

void snprintk_post (char *PostString)
{
   struct sockaddr_in local_addr;
   struct addrinfo *res;

   local_addr.sin_family = AF_INET;
   local_addr.sin_port = htons(0);
   local_addr.sin_addr.s_addr = 0;

    printk("HTTP example\n\r");
    int err = getaddrinfo(HTTP_HOST, NULL, NULL, &res);
    printk("get addrinfo\n\r");
    printk("getaddrinfo err: %d\n\r", err);
    ((struct sockaddr_in *)res->ai_addr)->sin_port = htons(HTTP_PORT);
    
   
    char send_buf[MAXLINE + 1];
    int send_data_len_post  = snprintk(send_buf,
				         500, 
				         POST_TEMPLATE, HTTP_PATH,
				         HTTP_HOST, strlen(PostString),
				         PostString);

     printf("send_buf is: %s \n",send_buf);
     printk("send err: %d\n\r", send_data_len_post);
     
    printk("socket\n\r");
     int client_fd = socket(AF_INET, SOCK_STREAM, 0);

     printk("client_fd: %d\n\r", client_fd);
     err = bind(client_fd, (struct sockaddr *)&local_addr, sizeof(local_addr));
     printk("bind err: %d\n\r", err);
     err = blocking_connect(client_fd, (struct sockaddr *)res->ai_addr,sizeof(struct sockaddr_in));
     printk("connect err: %d\n\r", err);

     int num_bytes = send(client_fd, send_buf, send_data_len_post, 0);

     printk("send err: %d\n\r", num_bytes);

     int tot_num_bytes = 0;

	do {
		num_bytes =blocking_recv(client_fd, recv_buf, RECV_BUF_SIZE, 0);
		tot_num_bytes += num_bytes;

		if (num_bytes <= 0) {
			break;
		}
                printk("Reply from the webserver \n\r");
		printk("%s", recv_buf);
	} while (num_bytes > 0);
        
}


void main(void)
{
      
	
      printk("Connecting LTE \n\r");
      printk("This may take several minutes.\n");
     
      int err = lte_lc_init_and_connect();
		if (err) {
			printk("LTE link could not be established.\n");
			
		}
      int err2 = lte_lc_psm_req(true);
		if (err2) {
			printk("LTE PSM could not be established.\n");
			
		}
      printk("Connected to LTE network\n");
       while(1){ 
         printk("HTTP to webserver \n");
         snprintk_post("HELLO from THE TESTING"); 
         k_sleep(2000); 
          
                 
	       }
       
     }